'''
Utility module that can be used to send the test proposal created message to the esb

@author: jasonmarshall
'''
from suds.client import Client
import logging

if __name__=='__main__':
    
    with open('test_message.xml', 'r') as message:
        
        
        logging.basicConfig(level=logging.INFO)
        
        test_event = message.read()
        client = Client('http://localhost:7789/?wsdl', cache=None)
        success = client.service.pushEvent(test_event)
        
        if success:
            logging.info('Message pushed succesfully')
        else:
            logging.error('Message push FAILED!')



from rmas_pm_adapter.pm_api import set_ethics_approved, get_proposal_for_rmas_id
import logging
def parse_ethics_approved_payload(payload):
    '''
    A sample payload looks like:

    <CERIF
        xmlns="urn:xmlns:org:eurocris:cerif-1.4-0" 
        xsi:schemaLocation="urn:xmlns:org:eurocris:cerif-1.4-0http://www.eurocris.org/Uploads/Web%%20pages/CERIF-1.4/CERIF_1.4_0.xsd" 
        xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
        release="1.4"
        date="2012-04-12"
        sourceDatabase="OpenEthics">
        
        <cfProj>
            <cfProjId>%(rmas_id)s</cfProjId> <!-- RMAS identifier --> 
            <cfProj_Class>
                <cfClassId>11111-11111-11111-11111</cfClassId><!-- this is the uuid for the status "Ethics Approved" -->
                <cfClassSchemeId>759af93a-34ae-11e1-b86c-0800200c9a66</cfClassSchemeId><!--this is the uuid for the CERIF scheme "Activity Statuses"-->
                <cfStartDate>%(start)s</cfStartDate>
                <cfEndDate>%(end)s</cfEndDate>
            </cfProj_Class>
        </cfProj>
    </CERIF> 

    At this stage we are only interested in getting the project id.... we could
    check to make sure that there is a node with the ethics_approved status
    but by virtue of the fact that this is an ethics_approved message we will
    trust that the sender has sent it legitimately.
    
    Returns:
    
    The proposal id.
    '''
    
    projid = payload.xpath('p:cfProj/p:cfProjId', 
                              namespaces={'p':'urn:xmlns:org:eurocris:cerif-1.4-0'}).pop().text
    return projid

def handle_event(payload):
    '''
        Handles the ethics_approved event.
        
        It will attempt to set the ethics_approved flag for the given proposal
        to True
    '''
    
    #parse the payload
    proposal_id = parse_ethics_approved_payload(payload)
    
    proposal = get_proposal_for_rmas_id(proposal_id)
    try:
        #update the proposal to set ethics_approved as true
        set_ethics_approved(proposal['id'])
    except ValueError as e:
        logging.info(e)
        


import requests
import json
from rmas_adapter.conf import settings
import logging
    
    
def get_proposal_for_rmas_id(rmas_id):
    '''
        Gets the proposal details based on the rmas_id.
        
        The is would look like:
        
        curl --request GET 'http://0.0.0.0:3000/proposals.json?rmas_id=urn:rmas:kent:pmtool:2edbb8d0-6ebe-4bec-912a-88fea00c0000'
    '''
    
    proposal_request = requests.get(settings.PM_API_PROPOSAL_ENDPOINT + '.json',
                                           params={'rmas_id':rmas_id})
    
    if proposal_request.status_code == 200:     
        #there should only be one resource returned:
        proposals = proposal_request.json
        
        if len(proposals) > 0:
            proposal = proposals[0]
            logging.info('Got Proposal: %s' % proposal)
            return proposal
        
    return None #no proposal for this rmas_id
    
def set_ethics_approved(proposal_id):
    '''
        Attempts to set the ethics_approved flag to True for the supplied
        proposal id.
        
        If the proposal doesn't exist then this will throw a ValueError.
        
        Doing this manually looks like:
        
        curl -v -H "Accept: application/json" -H 
        "Content-type: application/json" -X PUT 
        -d ' {"proposal":{"ethics_approved":"true"}}' http://0.0.0.0:3000/proposals/1.json
    '''
    headers = {'content-type': 'application/json', 'Accept':'application/json'}
    data = json.dumps({'proposal':{'ethics_approved':True}})
    
    new_application_request = requests.put(settings.PM_API_PROPOSAL_ENDPOINT +'/' +str(proposal_id) + '.json', 
                                           data=data, 
                                           headers=headers)
    
    if new_application_request.status_code == 200: 
        logging.info('Sucesfully set the ethics_approved flag for id: %s' % str(proposal_id))
    else:
        logging.info('FAILED to set the ethics_approved flag for id: %s' % str(proposal_id))
        
        
        
        


import os
import logging

if __name__ == '__main__':

    #this has to come first before you import any other modules otherwsise the settings won't be intitialized
    os.environ.setdefault("RMAS_ADAPTER_SETTINGS", "settings")
    
    logging.basicConfig(level=logging.INFO)
    
    from rmas_adapter.core.poller import start_polling
    start_polling()





RMAS_BUS_WSDL='http://localhost:7789/?wsdl'
POLL_INTERVAL=5000
EVENTS=[
        ('ethics-approved', 'handlers.ethics_approved'),
        ]


PM_API_PROPOSAL_ENDPOINT = 'http://0.0.0.0:3000/proposals'


from distutils.core import setup

setup(
    name='RMAS-PM-Adapter',
    version='0.1.0',
    author='Jason Marshall',
    author_email='j.j.marshall@kent.ac.uk',
    packages=['rmas_pm_adapter', 
              'rmas_pm_adapter.handlers', 
              'rmas_pm_adapter.dev'
              ],
    url='http://pypi.python.org/pypi/RMAS-OE-Adapter/',
    license='LICENSE.txt',
    description='A basic framework for building RMAS adapters',
    long_description=open('README.md').read(),
    install_requires=[
        "requests == 0.14.1",
        "RMASAdapter == 0.1.4 ",
    ],
)